@@ -3,6 +3,9 @@
     :codeauthor: :email:`Mike Place <mp@saltstack.com>`
 '''
 
+# Import python libs
+from __future__ import absolute_import
+
 # Import Salt Testing libs
 from salttesting import TestCase, skipIf
 from salttesting.helpers import ensure_in_syspath
@@ -14,6 +17,11 @@ import integration
 from salt import client
 from salt.exceptions import EauthAuthenticationError, SaltInvocationError, SaltClientError
 
+if integration.SaltClientTestCaseMixIn().get_config('minion')['transport'] != 'zeromq':
+    NOT_ZMQ = True
+else:
+    NOT_ZMQ = False
+
 
 @skipIf(NO_MOCK, NO_MOCK_REASON)
 class LocalClientTestCase(TestCase,
@@ -57,6 +65,7 @@ class LocalClientTestCase(TestCase,
                                                 kwarg=None, expr_form='list',
                                                 ret=['first.func', 'second.func'])
 
+    @skipIf(NOT_ZMQ, 'This test only works with ZeroMQ')
     def test_pub(self):
         # Make sure we cleanly return if the publisher isn't running
         with patch('os.path.exists', return_value=False):
