@@ -9,7 +9,12 @@ import string
 
 # Import Salt Testing libs
 from salttesting.unit import skipIf, TestCase
-from salttesting.mock import NO_MOCK, NO_MOCK_REASON, patch
+from salttesting.mock import (
+    MagicMock,
+    NO_MOCK,
+    NO_MOCK_REASON,
+    patch
+)
 from salttesting.helpers import ensure_in_syspath
 
 ensure_in_syspath('../../')
@@ -19,15 +24,13 @@ import salt.config
 import salt.loader
 from salt.modules import boto_lambda
 from salt.exceptions import SaltInvocationError
+from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 
 # Import 3rd-party libs
 from tempfile import NamedTemporaryFile
 import logging
 import os
 
-# Import Mock libraries
-from salttesting.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch
-
 # pylint: disable=import-error,no-name-in-module
 try:
     import boto3
